function xdata_adjusted = c166_fixpt_evenspace_cleanup(xdata_original,xdt,xscale)
%C166_FIXPT_EVENSPACE_CLEANUP adjust fixed point data for even spacing
%XDATA_ADJUSTED = C166_FIXPT_EVENSPACE_CLEANUP(XDATA_ORIGINAL,XDT,XSCALE)
%   This function modifies lookup table input data to be be evenly spaced if is
%   not quite evenly spaced after quantization.  For example, 0:0.005:1 appears
%   to be evenly spaced, but if it is quantized with scaling 2^-12, it will not
%   be evenly spaced.  Loss of even spacing can make a very significant impact
%   on the efficiency of the implementation.  Code generated by Real Time
%   Workshop to implement an uneven lookup table will be more complicated, and
%   equally important, the the input data will be stored in ROM.  Modifying the
%   input data to remain evenly spaced after quantization allows Real Time
%   Workshop to generate simpler code and to exclude the input data from memory
%   thereby saving significant amounts of ROM.  The modifications to the lookup
%   table input data are likely to change the numerical behavior of the table.
%   The numerical changes may or may not be trivial, so the model should be
%   tested using simulation, rapid prototyping, or other appropriate methods.
%
%   xdata_original is the input lookup data, ex. 0:0.005:1
%   xdt            is the input data type,   ex. sfix(16)
%   xscale         is the input scaling,     ex. 2^-12
%

% Copyright 1994-2002 The MathWorks, Inc.
% $Revision: 1.1.4.1 $  
% $Date: 2004/04/16 22:19:49 $

xdiff = diff(xdata_original);

nx = length(xdata_original);

diffdelta = max(xdiff) - min(xdiff);

xdata_adjusted = xdata_original;

if diffdelta < xscale(1)/2
  
  % The spacing in the ideal real world version of
  % the xdata_original is even or nearly even.  Nearly even
  % means that any difference in spacing is less
  % than half the fixed point precision.
  
  xdata_quantized = num2fixpt( xdata_original, xdt, xscale, 'Nearest', 'on' );
  
  temp = diff(xdata_quantized);

  diffdelta_quantized = max(temp) - min(temp);

  if diffdelta_quantized > 0
    
    % After quantization the scaling was not evenly spaced
    % Corrective action to restore the even spacing will be
    % applied.
    
    % Do NOT include bias when quantizing the delta
    %
    mean_diff_quantized = num2fixpt( mean(xdiff), xdt, xscale(1), 'Nearest', 'on' );
    
    xstart_quantized = num2fixpt( xdata_original(1), xdt, xscale, 'Nearest', 'on' );
    
    xdata_adjusted = xstart_quantized + (0:(nx-1))*mean_diff_quantized;
    
  end
end
